# -*- coding: utf-8 -*-
# Copyright (c) 2020, Frappe Technologies Pvt. Ltd. and contributors
# For license information, please see license.txt

from __future__ import unicode_literals
import frappe
import re
import pytz
from frappe.model.document import Document
from frappe import _
from datetime import datetime
from six import string_types
from pyyoutube import Api

class Video(Document):
	def validate(self):
		self.set_video_id()
		self.set_youtube_statistics()

	def set_video_id(self):
		if self.provider == "YouTube" and self.url and not self.get("youtube_video_id"):
			self.youtube_video_id = get_id_from_url(self.url)

	@classmethod
	def set_youtube_statistics(self, video_ids=None, update=True):
		if self.provider == "YouTube" and not is_tracking_enabled():
			return

		api_key = frappe.db.get_single_value("Video Settings", "api_key")
		api = Api(api_key=api_key)

		try:
			video_id = video_ids or self.youtube_video_id
			video = api.get_video_by_id(video_id=video_id)

			if video_ids:
				video_stats = video.items
			else:
				video_stats = video.items[0].to_dict().get('statistics')

			if not update:
				return video_stats

			self.like_count = video_stats.get('likeCount')
			self.view_count = video_stats.get('viewCount')
			self.dislike_count = video_stats.get('dislikeCount')
			self.comment_count = video_stats.get('commentCount')

		except Exception:
			title = "Failed to Update YouTube Statistics for Video: {0}".format(self.name)
			frappe.log_error(title + "\n\n" +  frappe.get_traceback(), title=title)

def is_tracking_enabled():
	return frappe.db.get_single_value("Video Settings", "enable_youtube_tracking")

def get_frequency(value):
	if not value:
		return None

	# Return frequency in hours
	if value != "Daily":
		return frappe.utils.cint(value[:2].strip())
	else:
		# 24 hours for Daily
		return 24


def update_youtube_data_half_hourly():
	# Called every 30 mins via hooks
	frequency = get_frequency(frappe.db.get_single_value("Video Settings", "frequency"))
	if not is_tracking_enabled() or not frequency:
		return

	if frequency == 30:
		batch_update_youtube_data()


def update_youtube_data():
	# Called every hour via hooks
	frequency = get_frequency(frappe.db.get_single_value("Video Settings", "frequency"))

	# if frequency is 30 mins dont proceed, as its handled in another method
	if not is_tracking_enabled() or not frequency or frequency == 30:
		return

	time = datetime.now()
	timezone = pytz.timezone(frappe.utils.get_time_zone())
	site_time = time.astimezone(timezone)

	if site_time.hour % frequency == 0:
		batch_update_youtube_data()

def get_formatted_ids(video_list):
	# format ids to comma separated string for bulk request
	ids = []
	for video in video_list:
		ids.append(video.youtube_video_id)

	return ','.join(ids)

@frappe.whitelist()
def get_id_from_url(url):
	"""
		Returns video id from url
		:param youtube url: String URL
	"""
	if not isinstance(url, string_types):
		frappe.throw(_("URL can only be a string"), title=_("Invalid URL"))

	pattern = re.compile(r'[a-z\:\//\.]+(youtube|youtu)\.(com|be)/(watch\?v=|embed/|.+\?v=)?([^"&?\s]{11})?')
	id = pattern.match(url)
	return id.groups()[-1]


@frappe.whitelist()
def batch_update_youtube_data():
	def prepare_and_set_data(video_list):
		video_ids = get_formatted_ids(video_list)
		Video.provider = "YouTube"
		stats = Video.set_youtube_statistics(video_ids=video_ids, update=False)
		set_youtube_data(stats)

	def set_youtube_data(entries):
		for entry in entries:
			video_stats = entry.to_dict().get('statistics')
			video_id = entry.to_dict().get('id')
			stats = {
				'like_count' : video_stats.get('likeCount'),
				'view_count' : video_stats.get('viewCount'),
				'dislike_count' : video_stats.get('dislikeCount'),
				'comment_count' : video_stats.get('commentCount')
			}

			frappe.db.sql("""
				UPDATE `tabVideo`
				SET
					like_count  = %(like_count)s,
					view_count = %(view_count)s,
					dislike_count = %(dislike_count)s,
					comment_count = %(comment_count)s
				WHERE youtube_video_id = '{0}'""".format(video_id), stats)

	video_list = frappe.get_all("Video", fields=["youtube_video_id"])
	if len(video_list) > 50:
		# Update in batches of 50
		start, end = 0, 50
		while start < len(video_list):
			batch = video_list[start:end]
			prepare_and_set_data(batch)
			start += 50
			end += 50
	else:
		prepare_and_set_data(video_list)